%{
===============================================================================
DESIGNED BY:    Tsezar F. Seman
AFFILIATION:    NJIT University, Physics Department
MODIFIED ON:    06/16/2012
COPYRIGHT:      (c) Tsezar F. Seman. All rights reserved.
PURPOSE:        Function initializes all static components, alocates all global
				variables. This function must be called ONCE at the begining.
INPUT:			N - system base => N^2 atoms
				CH - flag for Core-Hole presence
===============================================================================
%}
function Initialize(N,CH)
	if nargin < 0; error('ERR: Initialize(..) must have input parameters.'); end;
	if nargin == 1; isCluster = false; else isCluster = true; end;
	%------------------------------------------------------
	% DEFINE COMMON GLOBAL STRUCTURE
	%------------------------------------------------------
	global GL
	GL.path = MakeDir(sprintf('_N%02i_data_',N));
	% do not define GL for band plots
	if isCluster
		if mod(N,4) ~= 0 || N<4; error('ERR: System prameter N must be a positive integer divisible by 4.'); end;
		if mod(round(CH),9) ~= CH; error('ERR: Core-Hole parameter CH must be one of: {0,1,..,8}.'); end;
		GL.N = N;
		GL.CH = CH;
		GL.kPoints = GetAll_kPoints(GL.N);	% create all k-points in reduced Bruloin zone (N^2/8)
		GL.iPoints = GetAll_iPoints(GL.N);	% create all i-space points (N^2)
		GL.nXpt_list = zeros(GL.N^2, 4);	% allocate table for <n..>: 1:4->eg

		% constant matrices
		GL.H_nonint_eg = [];
		GL.H_corehole_eg = [];
		GL.H_tot_eg = [];
		GL.R_eg = [];					% used in Oper_n_eg() (see init below..)
		GL.kDiff_eg = Get_kDiff_eg();	% used in Oper_n_eg()
		
		% create/verify directory/file prefix for: {n-Operators, main data, tables}
		GL.file_nOper = sprintf('%s/nOper_%%04i.mat', MakeDir(sprintf('_nOper_N%02i_',GL.N)) );
		GL.file_main = sprintf('%s/N%02i_CH%i', MakeDir([GL.path,'/_main_']), GL.N, GL.CH);
		GL.file_tables = sprintf('%s/N%02i_CH%i', MakeDir([GL.path,'/_tables_']), GL.N, GL.CH);
		GL.file_ndiff = sprintf('%s/N%02i_CH%i', MakeDir([GL.path,'/_nDiff_']), GL.N, GL.CH);
		GL.file_rixs = sprintf('%s/N%02i_CH%i', MakeDir([GL.path,'/_RIXS_']), GL.N, GL.CH);
		
		% miscellaneous
		GL.figId = 1;			% starting figId
		GL.timecpu = cputime;	% cpu time to calculate GL.timediff
		GL.timediff = 0;		% time that already ellapsed in sec
		GL.Track = clDataTrack(5);	% tracking data: [E_occ, nXpt(0,0,0)]
		GL.Track_iPoint = find(GL.iPoints(:,1)==0 & GL.iPoints(:,2)==0);
		if isempty(GL.Track_iPoint); error('ERR: Suggested tracking (CH) site is not within the domain of iPoints.');end;
		
		GL.isPullay = 0;	% {0,1,2}
		%{-
		if mod(CH,2) == 1
			GL.isPullay = 2;
		else
			GL.isPullay = 0;
		end
		%}		
		if GL.isPullay == 1
			GL.alphaPullay = 0.1;
			if GL.alphaPullay > 1.0; error('ERR: alpha > 1, must be < 1.'); end;
			GL.nXpt_list(:,1) = 0.25;
			GL.nXpt_list(:,3) = 0.25;
			GL.nXpt_list_old = GL.nXpt_list;	% allocate table for <n..>   USED for Pullay Mixing only!
		elseif GL.isPullay == 2
			GL.alphaPullay = 0.3;
			GL.betaPullay = 0.3;
			if (GL.alphaPullay+GL.betaPullay) > 1.0; error('ERR: alpha+beta > 1, must be < 1.'); end;
			GL.nXpt_list(:,1) = 0.25;
			GL.nXpt_list(:,3) = 0.25;
			GL.nXpt_list_old = GL.nXpt_list;
			GL.nXpt_list_old2 = GL.nXpt_list;
		end
	end
	
	%------------------------------------------------------
	% DEFINE Eg GLOBAL STRUCTURE
	%------------------------------------------------------
	global Eg
	% Physical Constants:
	%Eg.C.lambda = sqrt(3/2)*1.38*0.4/0.228;		% (eV/angstrom), J-T param, (= 2.9652)
%	Eg.C.lambda = 1.69;							% (eV/angstrom), J-T param
%	Eg.C.JhSc = 2.47 / 2;						% (eV), Hund's param, (= 1.235)
%	Eg.C.U = 1.6;								% (eV), Coulomb potential
%	Eg.C.Uch = -2.6;							% (eV), core-hole potential

	% Experimentally determined:
	Eg.C.Q_1s =  0.05311622;					% (A) distortion prarmenter
	Eg.C.Q_2s =  0.05444722;					% (A) distortion prarmenter
	Eg.C.Q_3u =  0.10736930;					% (A) distortion prarmenter
	Eg.C.Q_3s = -0.01183920;					% (A) distortion prarmenter
	
	% Resonance parameters (used in RIXS model only):
	%Eg.C.omega_in =	0.0;	% (eV), calculated later
	Eg.C.Gamma1 =	0.75;	% (eV), set here for Mn3 C-H
	Eg.C.Gamma2 =	0.75;	% (eV), set here for Mn4 C-H
	
	
	% CHOSEN PARAMETERS:
	Eg.C.beta = 2.0;							% (unitless) breathing parameter
	Eg.C.t0 = 1.2;
	Eg.C.lambda =  4.566; % 5.953;
	Eg.C.U = 0.95 * 1.6;  %(= 1.52)
	Eg.C.JhSc = 1.8 * 2.47 / 2;  % (=2.2230)
	Eg.C.Uch = -2.0;
	
	Eg.C.shift = 4.17279306;	% value for N16 t0 = 0.9
	

	%	beta	t0      lambda  U       Uch
	%----------------------------------------
	Params = [...
		1.5,	0.1,	10.79,	3.5,	-4.0,	4.64902595; ...	% 1		2.00 eV - R 4.01 - Erixs 2.005
		1.5,	0.6,	9.606,	3.5,	-4.0,	4.50117621; ...	% 2		1.90 eV - R 4.65 - Erixs 2.01
		1.5,	0.9,	7.407,	3.5,	-4.0,	4.17279306; ...	% 3		1.70 eV - R 5.98 - Erixs 2.035
		1.5,	1.1,	4.810,	3.5,	-4.0,	3.73921945; ...	% 4		1.50 eV - R 11.23 - Erixs 2.01
		1.5,	1.3,	3.800,	3.5,	-4.0,	3.61694785; ...	% 5		???? eV - R 10.94 - Erixs 2.171
		1.5,	1.32,	3.760,	3.5,	-4.0,	3.61736508; ...	% 6		???? eV - R 10.94 - Erixs 2.171
		1.5,	1.33,	3.748,	3.5,	-4.0,	3.61938530; ...	% 7		???? eV - R 10.94 - Erixs 2.171
		1.5,	1.35,	3.725,	3.5,	-4.0,	3.62362752; ...	% 8		???? eV - R 10.94 - Erixs 2.171
		1.5,	1.40,	3.650,	3.5,	-4.0,	3.62999964; ...	% 9		???? eV - R 10.94 - Erixs 2.171
		1.5,	1.5,	3.509,	3.5,	-4.0,	3.64416614; ...	% 10	1.70 eV
		];
		%1.5,	1.45,	3.575,	3.5,	-4.0,	0;
pCase = 3;

	Eg.C.beta	= Params(pCase,1);
	Eg.C.t0		= Params(pCase,2);
	Eg.C.lambda = Params(pCase,3);
	Eg.C.U		= Params(pCase,4);
	Eg.C.Uch	= Params(pCase,5);
	Eg.C.shift	= Params(pCase,6);
	
	
	
	% Enable ONLY for running: run_Uch() and Uch_Estimation()
	%Eg.C.Uch = -1 * load('_temp_.txt');
	
	%------------------------------------------------------
    % Hamiltonians 16x16:
	Eg.H_hund = SetHund_eg(Eg.C.JhSc);
	Eg.H_jt = SetJahnTeller_eg(Eg.C.lambda, Eg.C.Q_3u, Eg.C.Q_3s, Eg.C.Q_2s);
	Eg.H_brtng = SetBreathing_eg(Eg.C.lambda,Eg.C.beta, Eg.C.Q_1s);
	Eg.H_shift = Eg.C.shift .* eye(16);
	
	if isCluster
		GL.R_eg = Set_R_eg(Eg.C.Q_3u, Eg.C.Q_3s, Eg.C.Q_2s);
	end

	%------------------------------------------------------
	% FOR BANDS ONLY
	%------------------------------------------------------
	if not(isCluster)
		Eg.H_coloumb = SetCoulomb_Bands_eg(N);
	end
end
%==========================================================
function kPoints = GetAll_kPoints(N)
	kPoints = zeros(N*N/8, 2);
	dk = 2*pi/N;
	kx = dk*(-N/4+1:N/4);
	nkx = length(kx);
	i = 1;
	kPoints(i:i+nkx-1,1) = kx';
	i = i+nkx;
	ky = dk;	
	while nkx > 2
		kx = kx(2:nkx-1);
		nkx = length(kx);
		kPoints(i:i+nkx-1,1) = kx';
		kPoints(i:i+nkx-1,2) = ky;
		i = i+nkx;
		kPoints(i:i+nkx-1,1) = kx';
		kPoints(i:i+nkx-1,2) = -ky;
		i = i+nkx;
		ky = ky+dk;
	end
	kPoints = sortrows(kPoints,[2 1]);
end
%{
Function creates an array of N^2 equally spaced (in 3D) i-points symmetrical w.r.t (0,0). 
All dimensions run from -N/2+1 to N/2 with increments of 1.
NOTE: 3-rd dimension (3-rd column) are orbital id labels: {1,2,3},  not z-coordinate
%}
function iPoints = GetAll_iPoints(N)
	iPoints = zeros(N^2, 3);
	nx = -N/2+1:N/2;
	ny = nx;
	nn = length(nx);
	for iy = 1:nn
		i = (iy-1)*nn+1;
		iPoints(i:i+nn-1,1) = nx';
		iPoints(i:i+nn-1,2) = ny(iy);
	end
	% create orbital id labels: 1: Mn^{3+}_-,  2: Mn^{4+}, 3: Mn^{3+}_+
	for i = 1:size(iPoints,1)
		switch mod(-iPoints(i,2) - iPoints(i,1), 4)
			case 0	% Mn^{3+}_-
				iPoints(i,3) = 1;
			case 1	% Mn^{4+}
				iPoints(i,3) = 2;
			case 2	% Mn^{3+}_+
				iPoints(i,3) = 3;
			case 3	% Mn^{4+}
				iPoints(i,3) = 2;
		end
	end	
end
%==========================================================
function m16 = SetHund_eg(JhSc)
	Z = zeros(2);
	G = (JhSc / 2) * eye(2);
	m16 = [ ...
		 Z, G,-G, Z,   Z,-G,-G, Z; ...
		 G, Z, Z,-G,  -G, Z, Z,-G; ...
		-G, Z, Z, G,  -G, Z, Z,-G; ...
		 Z,-G, G, Z,   Z,-G,-G, Z; ...
		 Z,-G,-G, Z,   Z, G,-G, Z; ...
		-G, Z, Z,-G,   G, Z, Z,-G; ...
		-G, Z, Z,-G,  -G, Z, Z, G; ...
		 Z,-G,-G, Z,   Z,-G, G, Z ];
	m16(:,:,2) = -m16(:,:,1);
end

function m16 = SetJahnTeller_eg(lambda, Q_3u, Q_3s, Q_2s)
	Z = zeros(2);
	A = -lambda * Q_3u * [ 1,  0;  0, -1];
	B = -lambda * Q_3s * [ 1,  0;  0, -1];
	C = -lambda * Q_2s * [ 0, -1; -1,  0];
	m16 = [ ...
		A, Z, Z, B,   C, Z, Z, C; ...
		Z, A, B, Z,   Z, C, C, Z; ...
		Z, B, A, Z,   Z, C, C, Z; ...
		B, Z, Z, A,   C, Z, Z, C; ...
		C, Z, Z, C,   A, Z, Z, B; ...
		Z, C, C, Z,   Z, A, B, Z; ...
		Z, C, C, Z,   Z, B, A, Z; ...
		C, Z, Z, C,   B, Z, Z, A ];
end

function m16 = SetBreathing_eg(lambda, beta, Q_1s)
	Z = zeros(2);
	A = -lambda * beta * Q_1s * [1, 0; 0, 1];
	m16 = [ ...
		Z, Z, Z, A,   Z, Z, Z, Z; ...
		Z, Z, A, Z,   Z, Z, Z, Z; ...
		Z, A, Z, Z,   Z, Z, Z, Z; ...
		A, Z, Z, Z,   Z, Z, Z, Z; ...
		Z, Z, Z, Z,   Z, Z, Z, A; ...
		Z, Z, Z, Z,   Z, Z, A, Z; ...
		Z, Z, Z, Z,   Z, A, Z, Z; ...
		Z, Z, Z, Z,   A, Z, Z, Z ];
end
%==========================================================
%{
Creates a pair of 16x16 matrices used in Oper_n()
First 16x16 matrix is for kx component, second is for ky
%}
function m16z2 = Get_kDiff_eg()
	m16z2 = zeros(16,16,2);
	m2 = ones(2);
	del_kx = [0, pi,  0, pi,   -pi/2,  pi/2, -pi/2, pi/2];
	del_ky = [0,  0, pi, pi,   -pi/2, -pi/2,  pi/2, pi/2];
	for ir = 1:8
		iir = (ir-1)*2+1;
		for ic = 1:8
			iic = (ic-1)*2+1;
			m16z2(iir:iir+1, iic:iic+1, 1) = (del_kx(ic) - del_kx(ir)) .* m2;
			m16z2(iir:iir+1, iic:iic+1, 2) = (del_ky(ic) - del_ky(ir)) .* m2;
		end
	end
end

%{
Creates 3 pairs of 16x16 matrices used  in Oper_n()
There are 3 atom types {1,2,3}, and two orbital types each {-,+}
%}
function m16_R = Set_R_eg(Q_3u, Q_3s, Q_2s)
	if abs(Q_2s) > eps
		Q_33 = Q_3u + Q_3s;
		th0 = atan( (-Q_33 + sqrt(Q_33^2 + 4*Q_2s^2)) / (2*Q_2s) );
		theta = [-th0, 0, th0];
	else
		theta = [0, 0, 0];
	end
	
	% 1-2 dim is 16x16
	% 3-rd dim is atom type based on theta {1,2,3}	=> {Mn3_x, Mn4, Mn3_y}
	% 4-th dim is orbital {1,2} => {-,+}
	m16_R = zeros(16,16,3,2);
	
	c2 = cos(theta).^2;
	s2 = sin(theta).^2;
	cs = cos(theta).*sin(theta);

	ind = 1:2:16;
	for i = 1:3
		m16_R(ind,ind,		i,1) =  c2(i);
		m16_R(ind,ind+1,	i,1) =  cs(i);
		m16_R(ind+1,ind,	i,1) =  cs(i);
		m16_R(ind+1,ind+1,	i,1) =  s2(i);
		
		m16_R(ind,ind,		i,2) =  s2(i);
		m16_R(ind,ind+1,	i,2) = -cs(i);
		m16_R(ind+1,ind,	i,2) = -cs(i);
		m16_R(ind+1,ind+1,	i,2) =  c2(i);
	end
end
%==========================================================
% FOR BANDS ONLY:
function H_coulomb = SetCoulomb_Bands_eg(N_base)
	global GL  Eg
	
	NN = N_base^2;
	iPoints = []; nXpt_list = [];
	load(sprintf('%s/_main_/N%02d_CH0__nXpt_list',GL.path,N_base));
	nP = size(iPoints,1);
	
	% Create U-based list for Hartree-Fock
	U_list = zeros(size(nXpt_list));
	nXpt_sum = Eg.C.U * sum(nXpt_list(:,1:4),2);
	for ic = 1:4
		U_list(:,ic) = nXpt_sum - Eg.C.U * nXpt_list(:,ic);
	end
	
	m16_R = Set_R_eg(Eg.C.Q_3u, Eg.C.Q_3s, Eg.C.Q_2s);
	kDiff_eg = Get_kDiff_eg();
	
	spUp = 1; spDn = 2;
	oMn = 1; oPl = 2;
	H_coulomb = zeros(16,16,2);	
	for i = 1:nP
		U = U_list(i,:);
		vi = iPoints(i,1:2);
		iType = iPoints(i,3);
		m16_Exp = (1/NN) .* exp(1i.*( vi(1) .* kDiff_eg(:,:,1) + vi(2) .* kDiff_eg(:,:,2) ));
		
		H_coulomb(:,:,spUp) = H_coulomb(:,:,spUp) + ...
			m16_Exp .* ( U(1) .* m16_R(:,:,iType,oMn) + U(2) .* m16_R(:,:,iType,oPl) );
		
		H_coulomb(:,:,spDn) = H_coulomb(:,:,spDn) + ...
			m16_Exp .* ( U(3) .* m16_R(:,:,iType,oMn) + U(4) .* m16_R(:,:,iType,oPl) );
	end
end
%==========================================================




%{
	Params = [...
		2,	0.75,	5.953,	1.52,	-2; ...
		2,	1.2,	4.566,	1.52,	-2; ...
		1,	0.7,	10.13,	1.52,	-2; ...
		2,	0.05,	7.558,	1.52,	-2; ...
		2,	0.75,	5.953,	1.52,	-3.4; ...
		2,	1.2,	4.566,	1.00,	-2; ...
		];
%}
%{
	Params = [...
		2,	1.2,	4.566,	1.52,	-2; ...
		2,	1.2,	4.566,	1.52,	-1.75; ...
		2,	0.75,	5.953,	1.52,	-2; ...
		1.5, 1.0,	6.249,	1.52,	-2; ...		%N-set page N-6
		1.5, 1.2,	5.121,	1.52,	-2; ...
		1.5, 1.2,	5.121,	1.75,	-2; ...
		1.5, 1.1,	4.810,	1.52,	-2; ...		% 3-star CASE
		1.5, 1.1,	4.810,	1.52,	-1.8; ...
		1.5, 1.4,	4.723,	1.52,	-2; ...
		1.5, 1.4,	4.723,	1.52,	-2; ...
		1.5, 1.1,	4.810,	3.5,	-4.0; ...
		];
%}




























