% Entry point (Hit F5 to run)
%{
===============================================================================
DESIGNED BY:    Tsezar F. Seman
AFFILIATION:    NJIT University, Physics Department
MODIFIED ON:    04/10/2012
COPYRIGHT:      (c) Tsezar F. Seman. All rights reserved.
PURPOSE:		Program calculates band structure for La0.5_Sr1.5_MnO_3 using 
				tight binding model. Calculates energy band spectrum f 
				H_nonint at 0 K.
===============================================================================
%}
function run4_plot_band(N)
	clc; addpath('_functions_','_common_');
	if nargin < 1
		N = 16;
	end
	
	isSavePDF = 0;
	isSaveEPS = 0;
	isHighlight = 0;
	%------------------------------------------------------
	% INITIALIZATION:
	CmdTitle('LaSrMnO4 BAND STRUCTURE');
	Initialize(N);
	
	GridType = 2;	% {0,1,2,3}
	spin = 1;		% 1/2 => up/down
	SetGlobals(spin);
	[data,cLabels] = GetDataSet(GridType);
	%------------------------------------------------------
	% CALCULATE ALL ENERGIES:
	nk = size(data,1);
	% cols: 1:16 - eg data
	% 3-rd dim: 1 - w/o coloumb, 2 - w/ coloumb
	E = zeros(nk,16,2);
	for i = 1:nk
		k = data(i,2:3);
		% w/o coloumb
		E(i,1:16,1) = Bands_E_eg(k, spin, 0)';
		% w/ coloumb
		E(i,1:16,2) = Bands_E_eg(k, spin, 1)';
	end
	%------------------------------------------------------
	% PLOT BANDS:
	shift1 = -max(E(:,2,1));
	shift2 = -max(E(:,2,2));
	E(:,1:16,1) = E(:,1:16,1) + shift1;
	E(:,1:16,2) = E(:,1:16,2) + shift2;

	fprintf('H_coulomb:\nshift = %.8f\n',shift2);
	fprintf('Set: Eg.C.shift = %.8f\n', GetDelta(shift2));
	
	%minmax_E = [-1, ceil(max(max(E(:,16,:))))];
	minmax_E = [-1,11];
	orb2 = [max(E(:,2,1)), max(E(:,2,2))];
	orb3 = [min(E(:,3,1)), min(E(:,3,2))];
	
	fprintf('## PLOTING BANDS: ...\n');

	% PLOTTING H_nonint:
	PlotInit(cLabels, minmax_E, 0);
	PlotBands(data(:,1), E(:,1:16,1), 1);
	if isHighlight; Highlights(0); end
	
	% plotting min/max of the gap:
	minmax_x = get(gca, 'XLim');
	%line(minmax_x, orb2(1)*[1,1], 'Color','r');
	line(minmax_x, orb3(1)*[1,1], 'Color','r');

	% PLOTTING H_nonint+H_coulomb:
	FigInc();
	PlotInit(cLabels, minmax_E, 1);
	PlotBands(data(:,1), E(:,1:16,2), 1);
	if isHighlight; Highlights(1); end
	
	% plotting min/max of the gap:
	%line(minmax_x, orb2(2)*[1,1], 'Color','r');
	line(minmax_x, orb3(2)*[1,1], 'Color','r');

	% saving figures to PDF
	if isSavePDF
		fig2pdf(gcf-1,	sprintf('N%02d_CH0_Band_HF0_',N));
		fig2pdf(gcf,	sprintf('N%02d_CH0_Band_HF1_',N));
	end
	
	% PLOTTING H_nonint+H_coulomb for PRB:
	if GridType == 2
		bAddl = 1;
	else
		bAddl = 0;
	end
	%------------------------------------------------------
	% saving figures to EPS
	if isSaveEPS
		FigInc();
		PlotInit_PRB(cLabels, minmax_E);
		PlotBandsNonint_PRB(data(:,1), E(:,1:16,1));
		PlotBands_PRB(data(:,1), E(:,1:16,2));

		if bAddl
			% CALCULATE ADDITIONAL ENERGIES:
			[data2,cLabels2] = GetDataSet(3);
			nk2 = size(data2,1);
			E1 = zeros(nk2,16);
			E2 = zeros(nk2,16);
			for i = 1:nk2
				k = data2(i,2:3);
				% w/ couloumb
				E1(i,1:16) = Bands_E_eg(k, spin, 0)';
				% w/ couloumb
				E2(i,1:16) = Bands_E_eg(k, spin, 1)';
			end
			E1(:,1:16) = E1(:,1:16) + shift1;
			E2(:,1:16) = E2(:,1:16) + shift2;

			xPos = max(data(:,1));
			minmax_x = get(gca,'XLim');
			minmax_x(2) = minmax_x(2)+max(data2(:,1));
			set(gca,'XLim', minmax_x);
			% w/o coulomb
			PlotBandsNonint_PRB(xPos+data2(:,1), E1(:,1:16));
			% w/ coulomb
			PlotBands_PRB(xPos+data2(:,1), E2(:,1:16));

			FigText(xPos+cLabels{1,2},minmax_E(1), cLabels2{1,1}, 20, 0, 'left','top');
			FigText(xPos+cLabels{2,2},minmax_E(1), cLabels2{2,1}, 20, 0, 'right','top');

			set(gca,'PlotBoxAspectRatio',[1 1 1])
		else
			set(gca,'PlotBoxAspectRatio',[.25 1 1])
		end
		
		fig2eps(gcf,'bands');
		
	end
end

function SetGlobals(spin)
	global GL
	
	GL.figId = 10;
	GL.spin = spin;
	GL.cLine = {'-', '-'};
	GL.LineWidth = 2;
	GL.Color = [[0 0 1]; [1 0 0]];
end
function FigInc()
	global GL
	
	GL.figId = GL.figId + 1;
end
%==========================================================
% FOR BANDS PLOTTING ONLY:
function [data,cLabels] = GetDataSet(dataId)
	function str = AxisX(szLetter, kPoint)
		str = sprintf('%s\n(%s,%s)', szLetter, Num2Latex(kPoint(1),1), ...
			Num2Latex(kPoint(2),1));
	end
	% choose data set for bands
	switch dataId
		case 0
			% our custom, based on paper
			cSetIn = { ...
				'X',		[pi/4,pi/4,0]; ...
				'\Gamma',	[0,0,0]; ...
				'M',		[pi/2,0,0]; ...
				'X',		[pi/4,pi/4,0] };
		case 1
			cSetIn = { ...
				'\Gamma',	[0,0,0]; ...
				'M',		[pi/2,0,0]; ...
				};
		case 2
			% our custom, based on paper
			cSetIn = { ...
				'M',		[pi/2,0,0]; ...
				'Y',		[pi/4,-pi/4,0]; ...
				'\Gamma',	[0,0,0]; ...
				'M',		[pi/2,0,0]; ...
				'X',		[pi/4,pi/4,0]; ...
				'Y',		[pi/4,-pi/4,0]; ...
				};
		case 3
			% our custom, based on paper
			cSetIn = { ...
				'\Gamma',	[0,0,0]; ...
				'X',		[pi/4,pi/4,0]; ...
				};
		otherwise
			error('ERR: Improper Data ID.');
	end
	% create appropriate x-labels
	nC = size(cSetIn,1);
	cLabels = cell(size(cSetIn));
	for i=1:nC
		cLabels{i,1} = AxisX(cSetIn{i,1}, cSetIn{i,2});
	end
	% create parametrized data set of all k-points
	nn = 50;	% number of points per every segment
	data = zeros((nC-1)*nn+1,4);
	iik = 1:nn;
	offset = 0;
	for i=1:nC-1
		cLabels{i,2} = offset;		
		ka = cSetIn{i,2};
		kb = cSetIn{i+1,2};
		dk = norm(kb-ka)/nn;
		data(iik,1) = offset + (0:nn-1)*dk;
		kset = [linspace(ka(1),kb(1),nn+1); linspace(ka(2),kb(2),nn+1); linspace(ka(3),kb(3),nn+1)]';
		data(iik,2:4) = kset(1:nn,:);		
		offset = offset + dk*nn;
		iik = iik+nn;
	end
	i = i+1;
	cLabels{i,2} = offset;
	data(iik(1),1) = offset;
	data(iik(1),2:4) = kb;
end

function PlotInit(cLabels, minmax_E, bCoulomb)
	global GL
	
	szTitle = sprintf('Bands for La_{0.5}Sr_{1.5}MnO_4: H_{nonint}');
	if bCoulomb
		szTitle = sprintf('%s + H_{coulomb}',szTitle);
	end
	FigId(GL.figId, szTitle); hold on;

	clrBlk = [0 0 0];
	set(gca,'XTick',[]);
	nc = size(cLabels,1);
	for i = 1:nc
		x = cLabels{i,2};
		% vertical solid line
		plot([x,x],minmax_E,'-','Color',clrBlk,'LineWidth',1);
		% bottom text lablel
		text(x,minmax_E(1),cLabels{i,1}, 'FontSize',9, 'Color',clrBlk,...
			'HorizontalAlignment','center', 'VerticalAlignment','top');
	end	
%{
	% upper-left labels for data
	if GL.spin == 1; szSpin = '\uparrow'; else szSpin = '\downarrow'; end;
	x = cLabels{1,2} + 0.1;
	y = minmax_E(2) - 0.2;
	text(x,y,sprintf('e_{g} Bands (blue), spin %s',szSpin), ...
		'FontSize',10, 'HorizontalAlignment','left', 'Color',GL.Color(1,:));
%}
	hold off;
	% axis matters
	box on;
	set(gca,'YGrid','on');
	xlim([cLabels{1,2},cLabels{nc,2}]);
	ylim(minmax_E);
	title(szTitle);
	ylabel('Energy - E_F (eV)');
end

function PlotBands(vK,vE,iLine)
	global GL
	
	figure(GL.figId); hold on;
	for iBand = 1:size(vE,2)
		plot(vK, vE(:,iBand), GL.cLine{iLine}, 'LineWidth',GL.LineWidth, 'Color',GL.Color(iLine,:));
	end
	hold off;
end

function delta = GetDelta(shift)
	global Eg
	
	delta = Eg.C.shift + shift;
end
%==========================================================
% EPS - PRL
function PlotInit_PRB(cLabels, minmax_E)
	global GL

	FigId(GL.figId, 'PRB Bands'); hold on;

	FontSize = 25;
	clrBlk = [0 0 0];
	nc = size(cLabels,1);
	for i = 1:nc
		x = cLabels{i,2};
		% vertical solid line
		plot([x,x],minmax_E,'-','Color',clrBlk,'LineWidth',1);
		% bottom text lablel
		switch i
			case 1
				hAlign = 'left';
			case nc
				hAlign = 'right';
			otherwise
				hAlign = 'center';
		end
		text(x,minmax_E(1),cLabels{i,1}, 'FontSize',FontSize, 'Color',clrBlk,...
			'HorizontalAlignment',hAlign, 'VerticalAlignment','top');
	end	
	hold off;
	% axis matters
	box on;
	set(gca,'XTick',[]);
	set(gca,'YGrid','on');
	xlim([cLabels{1,2},cLabels{nc,2}]);
	ylim(minmax_E);
	ylabel('Energy (eV)', 'FontSize',FontSize);
	
	axis('tight');
%	axis('equal');
	set(gca, 'LineWidth',2, 'FontSize',FontSize);
	
	
end

function PlotBands_PRB(vK,vE)
	global GL
	
	figure(GL.figId); hold on;
	for iBand = 1:size(vE,2)
		plot(vK, vE(:,iBand), '-', ...
			'LineWidth',3, 'Color','k');
	end
	hold off;
end
function PlotBandsNonint_PRB(vK,vE)
	global GL
	
	figure(GL.figId); hold on;
	for iBand = 1:size(vE,2)
		plot(vK, vE(:,iBand), '--', ...
			'LineWidth',2, 'Color','b');
	end
	hold off;
end


%==========================================================
% HIGHLIGHT:
function Highlights(isCoulomb)
	global Eg

	height = 0.2;
	y0 = -0.0;
	colMn3p = [1 0 0];
	colMn3m = [.8 .5 0];
	colMn4p = [0 1 0];
	colMn4m = [0 .8 .5];
	
	yMn3 = Eg.C.lambda * sqrt(Eg.C.Q_30^2 + 4*Eg.C.Q_2s^2);
	yMn4 = Eg.C.lambda * abs(Eg.C.Q_30);
	yHund = 2*Eg.C.JhSc;
	yJT = yMn3 - yMn4;
	yT = Eg.C.t0;
	
	hold on;
	% Mn3+ bands
	if not(isCoulomb)
		% H_nonint
		DrawAlphaRect(y0,				height, colMn3m);
		DrawAlphaRect(y0+2*yMn3,		height, colMn3p);
		DrawAlphaRect(y0+yHund,			height, colMn3m);
		DrawAlphaRect(y0+yHund+2*yMn3,	height, colMn3p);
	else
		% H_nonint+H_coulomb
		yC = Eg.C.U;
		DrawAlphaRect(y0,				height, colMn3m);
		DrawAlphaRect(y0+2*yMn3+yC,		height, colMn3p);
		DrawAlphaRect(y0+yHund+yC,		height, colMn3m);
		DrawAlphaRect(y0+yHund+2*yMn3+yC,height, colMn3p);
	end
	% Mn4+ bands
	y0 = y0+yJT;
	if not(isCoulomb)
		% H_nonint
		DrawAlphaRect(y0,				height, colMn4m);
		DrawAlphaRect(y0+2*yMn4,		height, colMn4p);
		DrawAlphaRect(y0+yHund,			height, colMn4m);
		DrawAlphaRect(y0+yHund+2*yMn4,	height, colMn4p);
	else
		% H_nonint+H_coulomb
		DrawAlphaRect(y0,				height, colMn4m);
		DrawAlphaRect(y0+2*yMn4,		height, colMn4p);
		DrawAlphaRect(y0+yHund,			height, colMn4m);
		DrawAlphaRect(y0+yHund+2*yMn4,	height, colMn4p);
	end
	hold off;
end
function DrawAlphaRect(ypos,height,col)
	x = get(gca,'XLim');
	h = height/2;
	fill([x(1),x(2),x(2),x(1)], ypos+[-h,-h,h,h], col, ...
		'EdgeColor','none', 'FaceAlpha',0.3);
end

%==========================================================
% TEMP:
%==========================================================
function GetTheta( v3,v4,w3,w4 )
	iFlip = [1 2 3 4 5 6 7 8  15 16 13 14 11 12 9 10];
	ww = w3;
	w3 = w3(iFlip,1);
	w4 = w4(iFlip,1);
	
	%[ww, w3]

	cos_th = (w3' * v3 + w4' * v4) / (w3' * w3 + w4' * w4);
	sin_th = (w4' * v3 - w3' * v4) / (w3' * w3 + w4' * w4);
	tan_th = (w4' * v3 - w3' * v4) / (w3' * v3 + w4' * v4);
	
	th1 = acos(cos_th);
	th2 = asin(sin_th);
	th3 = atan2((w4' * v3 - w3' * v4), (w3' * v3 + w4' * v4));
	
	[th1, th2, th3]	
	th3*180/pi
	
	cos_th^2 + sin_th^2	
	w3'*w3 + w4'*w4
	
	
	[v3, (cos(th3).*w3 + sin(th3).*w4)]
	[v4, (-sin(th3).*w3 + cos(th3).*w4)]
	
	disp('--- Complex Part: ---')
	% calc Cab matrix:
	c33 = w3' * v3;
	c34 = w4' * v3;
	c43 = w3' * v4;
	c44 = w4' * v4;
	
	[[c33 c34];[c43 c44]]
	
	[v3, (c33.*w3 + c34.*w4)]
	[v4, (c43.*w3 + c44.*w4)]
end


% Irixs: column vector of 8 elements
function Irixs = GetIrixs(Vh, Ve, I)
	% collect non-zero values, in groups of 16
	for i = 1:16:size(Vh,1)
		ah = Vh(i:i+15,1);
		if norm(ah) ~= 0; break; end;
	end
	for i = 1:16:size(Ve,1)
		ae = Ve(i:i+15,1);
		if norm(ae) ~= 0; break; end;
	end
	% simple error reporting, just in case...
	if norm(ah) == 0 || norm(ae) == 0S
		disp([Vh,Ve]);
		error('ERR: GetIrixs(..) - Issues with eigen vectors.');
	end
	% re-shaping into 2x8 forms
	ah = reshape(ah,2,8);
	ae = reshape(ae,2,8);
	
	iah = 8 * ([1 2 3 4 5 6 7 8] - 1);
	iae = [ 1 2 3 4 5 6 7 8; ...	% id for: kh +(0,0)
			2 1 4 3 6 5 8 7; ...	% id for: kh +(pi,0)
			3 4 1 2 7 8 5 6; ...	% id for: kh +(0,pi)
			4 3 2 1 8 7 6 5; ...	% id for: kh +(pi,pi)
			5 6 7 8 4 3 2 1; ...	% id for: kh +(-pi/2,-pi/2)
			6 5 8 7 3 4 1 2; ...	% id for: kh +( pi/2,-pi/2)
			7 8 5 6 2 1 4 3; ...	% id for: kh +(-pi/2, pi/2)
			8 7 6 5 1 2 3 4];		% id for: kh +( pi/2, pi/2)
	
	aM = ae' * ah;
%Irixs = I * ones(8,1); return;
	Irixs = I * [ ...
		abs(sum( aM(iae(1,:)+iah) ))^2; ...
		abs(sum( aM(iae(2,:)+iah) ))^2; ...
		abs(sum( aM(iae(3,:)+iah) ))^2; ...
		abs(sum( aM(iae(4,:)+iah) ))^2; ...
		abs(sum( aM(iae(5,:)+iah) ))^2; ...
		abs(sum( aM(iae(6,:)+iah) ))^2; ...
		abs(sum( aM(iae(7,:)+iah) ))^2; ...
		abs(sum( aM(iae(8,:)+iah) ))^2 ];
end
