% Entry point (Hit F5 to run)
%{
===============================================================================
DESIGNED BY:    Tsezar F. Seman
AFFILIATION:    NJIT University, Physics Department
MODIFIED ON:    04/10/2012
COPYRIGHT:      (c) Tsezar F. Seman. All rights reserved.
PURPOSE:        Plotting various RIXS data.
===============================================================================
%}
function run6_plot_rixs_ext(N)
	clc; addpath('_functions_','_common_');
	if nargin < 1
		clear all;
		N = 16;		% must be divisible by 4
	end;
	isSavePDF = 0;
	isSaveEPS = 0;
	%--------------------------------------------
	% GENERATE RIXS DATASET:
	CmdTitle(sprintf('LaSrMnO4 PLOTTING RIXS: N = %i', N));
	fprintf('## LOADING CALCULATED RIXS: ... \n');
	LoadCalcRixs(N);

	%--------------------------------------------
	% PLOTTING:
	fprintf('\n## GENERATING PLOTS: ... \n');

	t0 = Get_t0();
	Turn_AVG_On();
	
	Plot_qwContour_Ext__EPS();	% Contour Smooth
	if isSaveEPS; fig2eps(gcf,'contour-ext', 0); end;	% eps base: 0-vector, 1-image
return;

	Plot_kMapExt();
	if isSavePDF; fig2pdf(gcf,sprintf('N%02i_p4_k-Intens_Avg_t%.2f_',N,t0)); end;
	if isSaveEPS; fig2eps(gcf,sprintf('k-map-ext-t%.2f',t0)); end;
%return
	Plot_VShapeExt();
	if isSavePDF; fig2pdf(gcf,sprintf('N%02i_p5_v-Shape_Avg_t%.2f_',N,t0)); end;
	if isSaveEPS; fig2eps(gcf,sprintf('v-line-ext-t%.2f',t0)); end;
end

function t0 = Get_t0()
	global GL
	
	t0 = GL.C.t0;
end
function Turn_AVG_On()
	global GL
	
	GL.figPref = sprintf('N=%i (Avg.: -q_x<->q_x)',GL.N);
	GL.RIXS_SUM = GL.RIXS_SUM_AVG;
end

%==========================================================
% PLOTTING:
function Plot_kMapExt()
	global GL
	
	rixs_sum = sum(GL.RIXS_SUM, 2);
	data = [GL.kPoints, rixs_sum];

	szTitle = sprintf('%s: \\int I_{RIXS} in extended k-space, t_0= %.2f eV', GL.figPref, GL.C.t0);
	
	dsq = pi / GL.N;	% half-width of the square color patch
	
	function SqPatch(x,y,dsq,col)
		arX = [x-dsq,x+dsq,x+dsq,x-dsq];
		arY = [y-dsq,y-dsq,y+dsq,y+dsq];
		fill(arX,arY,col, 'Linestyle','none');
	end
	
	GL.figId = FigId(GL.figId,szTitle);
	minmax_I = [min(data(:,3)), max(data(:,3))];
	
	nK = size(data,1);
	nC = 64;
	cmap = colormap( jet(nC) );
	hold on;
	for iPt = 1:nK
		q = (data(iPt,3) - minmax_I(1)) ./ (minmax_I(2) - minmax_I(1));
		% color
		iColor = max( 1, min(nC, round( (nC - 1) * q + 1 )) );
		SqPatch(data(iPt,1),data(iPt,2), dsq, cmap(iColor,:));
	end
	% draw grid with diamond frame
	PlotGrid();

	hold off;
	colorbar;
	caxis(minmax_I);
	axis('equal');
	xlim([-3*pi-dsq, 3*pi+dsq]);
	ylim([-3*pi-dsq, 3*pi+dsq]);
	box('on');
	title(szTitle);
	xlabel('\Delta{q_x}');
	ylabel('\Delta{q_y}');
	
	q = linspace(-3*pi,3*pi,7);
	[tickNum,tickLab] = GetLabels( q );
	set(gca, 'XTick',tickNum, 'XTickLabel',tickLab);
	set(gca, 'YTick',tickNum, 'YTickLabel',tickLab);	
	set(gca, 'TickDir','out');
end
function PlotGrid()
	lst = '--';
	lw = 1.5;
	colGrid = [1 1 1];
	% verticals
	plot(3*pi*[-1,-1], 3*pi*[-1,1], lst, 'Color',colGrid, 'LineWidth',lw);
	plot(  pi*[-1,-1], 3*pi*[-1,1], lst, 'Color',colGrid, 'LineWidth',lw);
	plot(  pi*[ 1, 1], 3*pi*[-1,1], lst, 'Color',colGrid, 'LineWidth',lw);
	plot(3*pi*[ 1, 1], 3*pi*[-1,1], lst, 'Color',colGrid, 'LineWidth',lw);
	% horizontals
	plot(3*pi*[-1,1], 3*pi*[-1,-1], lst, 'Color',colGrid, 'LineWidth',lw);
	plot(3*pi*[-1,1],   pi*[-1,-1], lst, 'Color',colGrid, 'LineWidth',lw);
	plot(3*pi*[-1,1],   pi*[ 1, 1], lst, 'Color',colGrid, 'LineWidth',lw);
	plot(3*pi*[-1,1], 3*pi*[ 1, 1], lst, 'Color',colGrid, 'LineWidth',lw);
	% center diamond
	plot([1,0,-1,0,1]*pi/2, [0,1,0,-1,0]*pi/2, lst, 'Color',colGrid, 'LineWidth',lw);
end

function Plot_VShapeExt()
	global GL
	
	rixs_sum = sum(GL.RIXS_SUM, 2);
	data = [GL.kPoints, rixs_sum];
	szTitle = sprintf('%s: V-lines in extended k-space, t_0= %.2f eV', GL.figPref, GL.C.t0);
	
	GL.figId = FigId(GL.figId,szTitle);
	I_lim = [0,max(data(:,3))];	
	data = sortrows(data, [2, 1]);
	
	% y val at k=(0,0)
	y0 = data(data(:,1)==0 & data(:,2)==0,3);
	yMax = zeros(4,1);
	
	hold on;
	% plot vertical markers
	plot(-sqrt(2)*pi*[1,1],I_lim, '--', 'Color',[0.5,0.8,1]);
	plot( sqrt(2)*pi*[1,1],I_lim, '--', 'Color',[0.5,0.8,1]);
	plot(-3*pi*[1,1],I_lim, '--', 'Color',[1,0.8,0.5]);
	plot(  -pi*[1,1],I_lim, '--', 'Color',[1,0.8,0.5]);
	plot(   pi*[1,1],I_lim, '--', 'Color',[1,0.8,0.5]);
	plot( 3*pi*[1,1],I_lim, '--', 'Color',[1,0.8,0.5]);
	
	% diagonal kx = ky:
	d = data(data(:,1) - data(:,2) == 0, :);
	x = sign(d(:,1)) .* sqrt(d(:,1).^2 + d(:,2).^2); y = d(:,3);
x_sp = x;
y_sp = y;
	hp1 = plot(x,y, '-*b');
	yMax(1) = max(y);
	% diagonal kx = -ky:
	d = data(data(:,1) + data(:,2) == 0, :);
	x = sign(d(:,1)) .* sqrt(d(:,1).^2 + d(:,2).^2); y = d(:,3);
	hp2 = plot(x,y, ':*b');
	yMax(2) = max(y);
	% vertical kx = 0:
	d = data(data(:,1) == 0, :);
	x = sign(d(:,2)) .* sqrt(d(:,1).^2 + d(:,2).^2); y = d(:,3);
	hp3 = plot(x,y, '-*r');
	yMax(3) = max(y);
	% horizontal ky = 0:
	d = data(data(:,2) == 0, :);
	x = sign(d(:,1)) .* sqrt(d(:,1).^2 + d(:,2).^2); y = d(:,3);
	hp4 = plot(x,y, ':*r');
	yMax(4) = max(y);
		
	hold off;
	cStr{1} = sprintf('R=%.2f; (-\\pi,-\\pi) <-> (\\pi,\\pi)', yMax(1)/y0);
	cStr{2} = sprintf('R=%.2f; (-\\pi,\\pi) <-> (\\pi,-\\pi)', yMax(2)/y0);
	cStr{3} = sprintf('R=%.2f; (0,-\\pi) <-> (0,\\pi)', yMax(3)/y0);
	cStr{4} = sprintf('R=%.2f; (-\\pi,0) <-> (\\pi,0)', yMax(4)/y0);
	
	legend([hp1,hp2,hp3,hp4],cStr{1},cStr{2},cStr{3},cStr{4},	'Location','SouthEast');
	title(szTitle);
	xlabel('Parametrized k_{xy}');
	ylabel('\Sigma I_{RIXS}');
	xlim([-1,1]*3*pi*sqrt(2));
	ylim(I_lim);
	box('on');
	
	
%{-
	% create spline data
	xFac = 0.5/(sqrt(2)*pi);
	xxSp = -1.5:0.01:1.5;
	yySp = spline(x_sp*xFac, y_sp, xxSp);
%	hold on; plot(xxSp/xFac,yySp, '-k',  'LineWidth',1); hold off;
	
	% extract SPLINE data
	[t0,lambda,beta,Uch] = GetParams();
	suf = sprintf('_t%0.3f_lambda%0.3f_beta%0.1f_Uch%.1f', t0,lambda,beta,Uch);
	
	MakeDir('_EPS_');
	file = sprintf('_EPS_/kline%s.txt',suf);
	fid = fopen(file, 'w');
	dataSmooth = [xxSp; yySp/yySp(xxSp==0.0)];
	fprintf(fid, '% .4f  %.15f\r\n', dataSmooth);
	fclose(fid);
	fprintf('    * File [%s] have been generated.\n',file);
%}
end
function [t0,lambda,beta,Uch] = GetParams()
	global GL
	
	t0 = GL.C.t0;
	lambda = GL.C.lambda;
	beta = GL.C.beta;
	Uch = GL.C.Uch;
end
%==========================================================
% FIGURE AID:
function [tickNum,tickLab] = GetLabels(val)
	tickNum = sort(val);
	for i=1:length(tickNum)
		tickLab{i} = Num2Latex(tickNum(i),0);	%#ok
	end
end
%==========================================================
% SAVING/LOADING:
function LoadCalcRixs(N)
	% ACCESS to overwrite
	global GL
	
	path = sprintf('_N%02i_data_/_RIXS_',N);
	load(sprintf('%s/N%02i_CalcRixs_Ext.mat',path,N));
	
	GL.figId = 30;
	GL.figPref = sprintf('N = %d',GL.N);
end
%==========================================================



%==========================================================
% LINE/POINT PLOTTING:
function index = KPtsToIndex(kPts)
	global GL
	
	del = 1e-10;
	index = zeros(size(kPts,1),1);
	for i = 1:size(GL.kPoints,1)
		k = GL.kPoints(i,:);
		tmp = kPts;
		tmp(:,1) = tmp(:,1) - k(1);
		tmp(:,2) = tmp(:,2) - k(2);
		index(sum(abs(tmp),2) < del, 1) = i;
	end
end
function index = KPtsToIndexExt(kPts)
	global GL
	
	del = 1e-10;
	index = zeros(size(kPts,1),1);
	for i = 1:size(GL.kPointsExt,1)
		k = GL.kPointsExt(i,:);
		tmp = kPts;
		tmp(:,1) = tmp(:,1) - k(1);
		tmp(:,2) = tmp(:,2) - k(2);
		index(sum(abs(tmp),2) < del, 1) = GL.kIndexExt(i,1);
	end
end

%==========================================================
function Plot_qwContour_Ext__EPS()
	global GL
	
	% kVals cols: 1 - parametric x value, 2 - vertical line flag, 3:4 - actual k-value
	sq = sqrt(2);
	p8 = pi/8;
	switch GL.N
		case 16
			gammaW = 0.6;
			
			% diagonal line
			kVals = [ ...
				[-sq*12*p8,		0,		-12*p8, -12*p8]; ...	% extended
				[-sq*11*p8,		0,		-11*p8, -11*p8]; ...	% extended
				[-sq*10*p8,		0,		-10*p8, -10*p8]; ...	% extended
				[-sq*9*p8,		0,		-9*p8, -9*p8]; ...	% extended
				[-sq*8*p8,		1,		-8*p8, -8*p8]; ...
				[-sq*7*p8,		0,		-7*p8, -7*p8]; ...
				[-sq*6*p8,		0,		-6*p8, -6*p8]; ...
				[-sq*5*p8,		0,		-5*p8, -5*p8]; ...
				[-sq*4*p8,		1,		-4*p8, -4*p8]; ...
				[-sq*3*p8,		0,		-3*p8, -3*p8]; ...
				[-sq*2*p8,		0,		-2*p8, -2*p8]; ...
				[-sq*1*p8,		0,		-1*p8, -1*p8]; ...
				[0, 			1,		0, 0]; ...
				[sq*1*p8,		0,		1*p8, 1*p8]; ...
				[sq*2*p8,		0,		2*p8, 2*p8]; ...
				[sq*3*p8,		0,		3*p8, 3*p8]; ...
				[sq*4*p8,		1,		4*p8, 4*p8]; ...
				[sq*5*p8,		0,		5*p8, 5*p8]; ...
				[sq*6*p8,		0,		6*p8, 6*p8]; ...
				[sq*7*p8,		0,		7*p8, 7*p8]; ...
				[sq*8*p8,		1,		8*p8, 8*p8]; ...
				[sq*9*p8,		0,		9*p8, 9*p8]; ...	% extended
				[sq*10*p8,		0,		10*p8,10*p8]; ...	% extended
				[sq*11*p8,		0,		11*p8,11*p8]; ...	% extended
				[sq*12*p8,		0,		12*p8,12*p8]; ...	% extended
				];
			PlotRixsContribSmoothQW_Fig__EPS(kVals,gammaW);
	end
end
function PlotRixsContribSmoothQW_Fig__EPS(kVals,gammaW)
	global GL
	
	FontSize = 20;
	Ny = 800;
	Nx = 200;
	
	LorentzianW = @(ww,w0) ( (gammaW./pi) ./ ((ww-w0).^2 + gammaW.^2) );
	
	nk = size(kVals,1);	
	minmax_q = [kVals(1,1), kVals(nk,1)];

	%minmax_w = [floor( min(GL.RIXS(:,4)) ), ceil( max(GL.RIXS(:,4)) )];
	%minmax_w = [0, ceil( max(GL.RIXS(:,4)) )];
	minmax_w = [0, 5];
	xCont = linspace(minmax_q(1), minmax_q(2), Nx);
	yCont = linspace(minmax_w(1), minmax_w(2), Ny);

	% k-point indexes to use in search for RIXS values
	kInd = KPtsToIndex(kVals(:,3:4));% + KPtsToIndexExt(kVals(:,3:4));
	
	% collect all w-sets as smooth Lorentzians
	w_set = zeros(Ny, nk);
	for ik = 1:nk
		% collect all intensities and energies for a given k-point
		IWrixs = GL.RIXS(GL.RIXS(:,8) == kInd(ik), 3:4);
		I_tmp = zeros(size(yCont));
		for iw = 1:size(IWrixs,1)
			I_tmp = I_tmp + IWrixs(iw,1) .* LorentzianW(yCont,IWrixs(iw,2));
		end
		w_set(:,ik) = I_tmp';
	end
		
	% interpolate horizontally
	[qq,ww] = meshgrid(xCont, yCont);
	zz = zeros(size(qq));
	q_param = kVals(:,1)';
	for iw = 1:Ny
		I_tmp = w_set(iw,:);
		zz(iw,:) = spline(q_param,I_tmp, xCont);
	end
	maxZ = max(max(zz));
	zz = zz ./ maxZ;
	
	% plot contour map
	szTitle = sprintf('%s: Integrated I_{ RIXS} Contribution',GL.figPref);
	GL.figId = FigId(GL.figId,szTitle);
	hold on;
	pcolor(qq,ww,zz);
	caxis([0, 1]);
	axis('tight');	%'equal'
	colormap(hot(128));
	shading('flat');
	hcb = colorbar;
	
%{-
	% EXTRA: collect max points for dispersion (Xuerong's request)
	Ipeaks = zeros(nk,3);	% (q_param, E, Imax)
	for ik = 1:nk
		[mI,ind] = max(w_set(:,ik));
		Ipeaks(ik,:) = [q_param(ik), yCont(ind), mI];
	end
	save('dispersion.txt','Ipeaks','-ascii','-double');
	plot(Ipeaks(:,1), Ipeaks(:,2), 'om');
%}

	% axis matters
	xlim(minmax_q);
	ylim(minmax_w);
	ylabel('Energy (eV)', 'FontSize',FontSize);
	set(gca, 'TickDir','out', 'FontSize',FontSize);
	
	pi2 = pi*sqrt(2);
	% vertical white lines
	line(-pi2*[1,1],minmax_w, 'Color',[1 1 1], 'LineStyle','--', 'LineWidth',2);
	line( pi2*[1,1],minmax_w, 'Color',[1 1 1], 'LineStyle','--', 'LineWidth',2);

	hold off;
	
	% colorbar adjustments
	set(hcb, 'LineWidth',2);
	set(hcb, 'YTick',[0,0.25,0.5,0.75,1], ...
		'YTickLabel',{'0';'';'0.5';'';'1'})
	

%	set(gca, 'XTick',[-pi2,-pi2/2,0,pi2/2,pi2], 'XTickLabel','');
	set(gca, 'XTick',[-3*pi2/2,-pi2,-pi2/2,0,pi2/2,pi2,3*pi2/2], ...
		'XTickLabel',{'-0.75';'-0.5';'-0.25';'0';'0.25';'0.5';'0.75'});
	set(gca, 'YTick',0:8, ...
		'YTickLabel',{'0';'';'2';'';'4';'';'6';'';'8'});
	xlabel('H (r.l.u.)');
	set(gca, 'LineWidth',2);
	%axis('square');
	set(gca,'PlotBoxAspectRatio',[1 .5 1])
	box('on')
	
	camva(1.2*camva)
end


































